function [DET,varargout]=fmmsre(P,A,B,C,R,Opt) %VERSION with FUll INFORMATION and inputs of A,B,C
%% This is a matlab code for implementing
%  Forward Method for MSRE (Narkov-switching rational expectations) models
%  Can be downloaded from http://web.yonsei.ac.kr/sc719/
%% Major Update: Sep 18, 2016 : Automatic detection of block-recursiveness 
%% References :
% Cho and Moreno(2011,JEDC), "Forward Method as a Solution Refinement in Rational Expectations Models"
% Cho(2016,RED) "Sufficient Conditions for Determinacy in a Class of 
%            Markov-Switching Rational Expectations Models" and its " A
%            Technical Guide"'
%% Usage: Specify the input arguments and run it
%       P,A (required) 
%       B,C,R and other optional arguments (optional).
% The code will produce 
%       DET : determinacy information, 
%       FCC : forward convergence information
%       OmegaK,GammaK,FK: Forward Solution
%       Other output such as Impulse Response Functions
% See below for detail.
% =========================================================================
%% Model and Solution
%   Model
%   B1(s(t))x(t)=E[A1(s(t),s(t+1))x(t+1)|I(t)]+B2(s(t))*x(t-1)+C1(s(t))*z(t)   
%           x(t)=E[A(s(t),s(t+1))x(t+1)|I(t)]+B(s(t))*x(t-1)+C(s(t))*z(t)
%           z(t)=R(s(t))*z(t-1)+e(t)
%           where x(t) is an n by 1 vector of endogenous variables
%                 z(t) is an m by 1 vector of exogenous variables
%                 s(t) is a S - state Markov chain.
%                 A(s(t),s(t+1))=B1(s(t))^(-1)*A1(s(t),s(t+1))
%                 B(s(t))=B1(s(t))^(-1)*B2(s(t))
%                 C(s(t))=B1(s(t))^(-1)*C1(s(t))
%   Solution 
%       x(t)=(s(t))*x(t-1)+(s(t))*z(t)+w(t), 
%       w(t)=E[F(s(t),s(t+1)w(t+1)|I(t)]
%%   [1] Input
%       [1-1] Required Input arguments
%           P   : S by S transition matrix where the ij-th element, 
%                 p_ij=Pr(s(t+1)=j|s(t)=i).
%           A   : S by S cell array such that A{i,j}=A(s(t)=i,s(t+1)=j), or
%                 S by 1 cell array such that A{i,1}=A(s(t)=i)
%                 ** size(A{i,j}) = n by n
%       [1-2] Optional Input arguments
%           B   : S by 1 cell array such that B{i,1}=B(s(t)=i). 
%                   - Default: B=zeros(n,n);
%                   - If there is no x(t-1) in the model, set B=[]; 
%           C   : S by 1 cell array such that C{i,1}=C(s(t)=i).: C=B0(s(t))^(-1) or C(s(t))=B0(s(t))^(-1)*C1(s(t) IS STRONGLY RECOMMENDED  
%                   - size(C{i,1})=n by m.
%                   - Default: C{i,1}=eye(n) for all i, so m=n; 
%                   - If C is not present, set C=[];
%           R   : m by m VAR coefficient matrix of z(t) or
%                 S by 1 cell array such that R{i,1}=R(s(t)=1). (NEW)
%                   - Default: R=zeros(m,m);
%                   - If R is zeros, set R=[]; 
%           Opt : Other option-structure
%               Opt.maxK: maximum number of forward recursion: 
%                   (Default:maxK=1000)
%                   Change this only if the code display a warning sign. 
%               Opt.tolK: Precision of Forward Convergence of Omega_k: 
%                   The forward iteration stops if the difference between 
%                   the largest element of Omega_k at all states in 
%                   absolute value at k and k-1 is less than Opt.tolK.
%                   (Default is 0.000001)
%               Opt.IRsigma : m by 1 (or 1 by m ) vector of standard errors
%                   of the structural shocks, e(t) used for generating IR 
%                   functions. (Default : m by 1 vector of ones)
%               Opt.IRT: number of period for IR functions. (Default=20)
%               Opt.IRvs: 
%                   Default arrangement : first variable to m shocks, ..., 
%                   the last variable to m shocks. 
%                   Set Opt.IRvs=1 if users want to arrange IR function 
%                   such that the n variables to the first shock, ..., n 
%                   variables to the last shock. 
%(NEW)          Opt.InfoAdj:  Whether to adjust information structure
%                   (Default is 1) Use full information in forming 
%                   expectations of the variables in autonomous block. 
%                   Set Opt.InfoAdj=0 to use the information in the autonomous
%                   block to form expectations of the variables in that block.
%                   (Opt.InfoAdj=0 was default in the previous forward
%                   method.)
%%   [2] Output : [DET,FCC,OmegaK,GammaK,FK, OtherInfo,IRS]
%       DET : [DET1 DET2]
%               DET1 = Spectral radius (SR) of BarPsi_(kron(_K,_K))
%                      : n^2*S by n^2*S 
%               DET2 = SR of Psi_(kron(F_K,F_K)) : n^2*S by n^2*S
%               ** See below for how to interpret DET.
%       FCC : [FCC1 FCC2]   
%               FCC1 = termK : number of forward iteration for Omega_K. 
%                   If termK < maxK, the FCC holds for Omega_K
%               FCC2 = R_Psi_RtkFK = SR of Psi_(kron(R',F_K)) : n*m*S by n*m*S  
%                   If R_Psi_RtkFK < 1, the FCC holds for Gamma_k
%       OmegaK  : S by 1 cell array such that OmegaK{i,1}=Omega_k(s(t)=i) 
%                   at "k=termK". If termK < maxK, OmegaK is Omega^*
%                   If termK = maxK, OmegaK is not the forward solution.
%       GammaK  : S by 1 cell array such that GammaK{i,1}=Gamma_k(s(t)=i) 
%                   at "k=termK". If FCC2 < 1, GammaK is Gamma^*
%                   If FCC2>=1, GammaK is not the forward solution.
%       FK      : S by S cell array such that FK{i,j}=F_k(s(t)=i,s(t+1)=j) 
%                   at "k=termK".
%       OtherOutput : Structure of Other output information 
%               OtherOutput.BarPsi_OKkOK : the matrix BarPsi_OKkOK
%               OtherOutput.Psi_FKkFK : the matrix Psi_FKkFK
%               OtherOutput.Psi_RtkFK : the matrix Psi_RtkFK
%               OtherOutput.ai: Index denoting autonomous block 
%       IRS : S by 1 cell array where i-th element of IRS is the IR of x(t)
%              when s(t)=i. It is T by n*m where each column is 
%               x(1)....x(n) to 1st shock,...  x(1)....x(n) to m-th shock 
%%   [3] Interpretation of DET and FCC
%%   [3-1] Determinacy and Classification of MSRE models
%   If FCC1<maxK, DET1<1 and DET2<=1, the model is determinate.
%   If FCC1<maxK, DET1<1 and DET2>1, type and run min_tau2. If tau2<1, 
%      the model is indeterminate. Refer to min_tau2.m for details.
%   If FCC<maxK, DET1>1 and DET2>1, check whether the model is block-recursive.
%      If so, adjust the model and run it again following Appendix C of
%      "Sufficient Conditions for Determinacy in a Class of Markov-Switching  
%      Rational Expectations Models: A Technical Guide".
%% [3-2] FCC and its interpretation
%   If termK = maxK, set a higher maxK. If termK=maxK always, then the
%      Omega^* does not exist. This will
%      happen when the model has no real-valued solution.
%   If FCC2=R_Psi_RtkFK<1, Gamma^* is well-defined. We compute this using 
%       the formula given in the paper instead of constructing this as a 
%       sequence to speed up computation. 
%       HOWEVER, If R_Psi_RtkFK>=1, Gamma^* does not exist. 
%       GammaK reported is NOT the forward solution.
% =========================================================================

%% Code
%% [0] Identify S (number of Regimes), n (rows of A) and 
%       m (columns of C)
    S=size(P,1); 
    n=size(A{1,1},1);
    if size(A,2)==1, 
        for i=1:S, for j=1:S, AA{i,j}=A{i,1}; end, end, A=AA; 
    end
  
    if nargin==2, B=[]; C=[]; m=n;    R=zeros(m,m);           end
    if nargin==3,       C=[]; m=n;    R=zeros(m,m);           end
    if nargin>3, if isempty(C), m=n; else  m=size(C{1,1},2);  end, end 
    if nargin==4,                     R=zeros(m,m);           end

%% [1] Make B and R cell arrays if they are not cell arrays. 
    if isempty(B),      for j=1:S, B{j,1}=zeros(n,n);       end, end  
    if isempty(C),      for j=1:S, C{j,1}=eye(n,n);         end, end  
    if isempty(R),      for j=1:S, R=zeros(m);              end, end  
%% [2] Option values  
    if nargin==6 && isfield(Opt,'maxK'), maxK=Opt.maxK; else maxK=1000; end
    if nargin==6 && isfield(Opt,'tolK'), 
                        tolK=Opt.tolK; else tolK=0.000001; end
    if nargin==6 && isfield(Opt,'IRsigma'), 
                        vecsig=Opt.IRsigma; else vecsig=ones(1,m); end   
    if nargin==6 && isfield(Opt,'IRT'), T=Opt.IRT; else T=20; end
    if nargin==6 && isfield(Opt,'IRvs'), vs=Opt.IRvs; else vs=0; end
    if nargin==6 && isfield(Opt,'InfoAdj'), InfoAdj=Opt.InfoAdj; else InfoAdj=1; end

%% [3] Forward Method
%% [3-1] Computing OmegaK, DET1 and FCC1 Under Partial Information 
    % Initial (s(t)=i) at k=1
        for i=1:S, OMEGAK{1,i}=B{i};  end,
    % Fill out Xi_k(s(t)),_k(s(t)), 
        k=0; FC_O=1;
        while FC_O>tolK
            k=k+1;   
            % (1) Xi{k,i}=eye(n)-E[A(s(t),s(t+1))*_{k-1}(s(t+1))| s(t)=i]         
                for i=1:S, 
                    EAOmegaki=zeros(n,n); 
                        for j=1:S,                 
                           EAOmegaki =EAOmegaki +P(i,j)*A{i,j}*OMEGAK{k,j};
                        end
                    Xi{k,i}=eye(n)-EAOmegaki;
                end

            % (4) Compute _{k}(s(t)) 
                FC_O_OmegaS=[]; 
                for i=1:S 
                    OMEGAK{k+1,i}=Xi{k,i}\B{i};
                    FC_O_OmegaS=[FC_O_OmegaS;...
                                max(abs(OMEGAK{k+1,i}(:)-OMEGAK{k,i}(:)))];
                end
            FC_O=max([FC_O_OmegaS]);     
            
            if k==maxK, 
            break; end  % End of k==maxK       
        end % end of "while".
        
        termK=k;   
    % Computing Omega_k(s(t)) and FK_k(s(t),s(t+1)) at k=termK
        for i=1:S, PI_OmegaK{i,1}=OMEGAK{end,i};  end  
        bdiagOm2=zeros(n^2*S,n^2*S);
        for i=1:S
            bdiagOm2(n^2*(i-1)+1:n^2*i,n^2*(i-1)+1:n^2*i)...
            = kron(PI_OmegaK{i},PI_OmegaK{i});
        end
        BarPsi_OKkOK= bdiagOm2*kron(P',eye(n^2));  %BarPsi_kron(OmegaK,OmegaK)
        PI_DET1=max(abs(eig(BarPsi_OKkOK)));
        PI_FCC1=termK;
        
%% [3-2] Computing OmegaK, DET1 and FCC1 Under  Full Information 
    if InfoAdj==1
        H=ones(n,n); % One can use randn(n,n)
        EB=cell(S,1);
        for i=1:S
            EBi=zeros(n,n); 
            for j=1:S              
                EBi =EBi +P(i,j)*B{j,1};
            end
            EB{i,1}=EBi;
        end
        
            % When H=ones(n,n), the BigB1 matrix may happen to be singular.
            % In this case, we set H=randn(n,n);
            rankBigB1=zeros(S,1);
            for i=1:S
                rankBigB1(i,1)=rank([eye(n) -eye(n);H*EB{i,1} eye(n)]);
            end
            if any(rankBigB1(i,1)-2*n*ones(S,1))~=0, H=randn(n,n); end

        for i=1:S
            BigB1{i,1}=[eye(n) -eye(n);H*EB{i,1} eye(n)];
            BigB2{i,1}=[B{i,1} zeros(n,n);zeros(n,2*n)];
            for j=1:S
                BigA1{i,j}=[zeros(n,2*n);A{i,j}+H -H];
                BigA{i,j}=BigB1{i,1}\BigA1{i,j}; 
            end
            BigB{i,1}=BigB1{i,1}\BigB2{i,1}; 
        end    
        N=2*n;

    % Initial (s(t)=i) at k=1
        for i=1:S, BigOMEGAK{1,i}=BigB{i,1};  end,
    % Fill out Xi_k(s(t)),_k(s(t)), 
        k=0; FC_O=1;
        while FC_O>tolK
            k=k+1;   
            % (1) Xi{k,i}=eye(n)-E[A(s(t),s(t+1))*_{k-1}(s(t+1))| s(t)=i]         
                for i=1:S, 
                    EAOmegaki=zeros(N,N); 
                        for j=1:S,                 
                           EAOmegaki =EAOmegaki +P(i,j)*BigA{i,j}*BigOMEGAK{k,j};
                        end
                    BigXi{k,i}=eye(N)-EAOmegaki;
                end

            % (4) Compute _{k}(s(t)) 
                FC_O_OmegaS=[]; 
                for i=1:S 
                    BigOMEGAK{k+1,i}=BigXi{k,i}\BigB{i};
                    FC_O_OmegaS=[FC_O_OmegaS;...
                                max(abs(BigOMEGAK{k+1,i}(:)-BigOMEGAK{k,i}(:)))];
                end
            FC_O=max([FC_O_OmegaS]);     
            
            if k==maxK, 
            break; end  % End of k==maxK       
        end % end of "while".
        
        BigtermK=k;   
    % Computing Omega_k(s(t)) and FK_k(s(t),s(t+1)) at k=termK
        for i=1:S, FI_OmegaK{i,1}=BigOMEGAK{end,i}(1:n,1:n);  end
        bdiagOm2=zeros(n^2*S,n^2*S);
        for i=1:S
            bdiagOm2(n^2*(i-1)+1:n^2*i,n^2*(i-1)+1:n^2*i)...
            = kron(FI_OmegaK{i},FI_OmegaK{i});
        end
        BarPsi_OKkOK = bdiagOm2*kron(P',eye(n^2)); %BarPsi_kron(OmegaK,OmegaK)
        FI_DET1=max(abs(eig(BarPsi_OKkOK)));
        FI_FCC1=BigtermK;      
    else
        FI_OmegaK=PI_OmegaK; 
        FI_DET1 = PI_DET1; 
        FI_FCC1=PI_FCC1;
    end % end of InfoAdj==1      
        
%% [3-3] Selection of OmegaK and FCC1 from two informational assumptions   
        if InfoAdj==1 && FI_DET1<=PI_DET1
            OmegaK=FI_OmegaK;
            DET1=FI_DET1;
            FCC1=FI_FCC1;  
        else 
            OmegaK=PI_OmegaK;
            DET1=PI_DET1;
            FCC1=PI_FCC1;
        end 
%% [3-4] Computing FK, DET and FCC  
    % (1) FK
               for i=1:S
                    EAOmegaK=zeros(n,n); 
                        for j=1:S               
                           EAOmegaK =EAOmegaK +P(i,j)*A{i,j}*OmegaK{j,1};
                        end
                    XiK{i,1}=eye(n)-EAOmegaK;
                end
        
        for i=1:S, for j=1:S, FK{i,j}=XiK{i,1}\A{i,j}; end, end
        
    % (2) DET2
        Psi_FKkFK=[]; 
        for i=1:S
            Psi_FKkFKrow=[]; 
            for j=1:S
                Psi_FKkFKrow=[Psi_FKkFKrow P(i,j)*kron(FK{i,j},FK{i,j})];
            end
            Psi_FKkFK=[Psi_FKkFK;Psi_FKkFKrow]; %Psi_(kron(FK,FK)) 
        end
        DET2=max(abs(eig(Psi_FKkFK))); 

        DET=[DET1 DET2]; %Determinacy Result

    % (3) FCC2    
        if iscell(R), RR=R;
        else for i=1:S, RR{i,1}=R; end
        end
    
        Psi_RtkFK=[]; 
        for i=1:S
            Psi_RtkFKrow=[]; 
            for j=1:S
                    Psi_RtkFKrow=[Psi_RtkFKrow P(i,j)*kron(RR{j,1}',FK{i,j})];
            end
            Psi_RtkFK=[Psi_RtkFK;Psi_RtkFKrow];  %Psi_(kron(R',FK)) 
        end
        FCC2=max(abs(eig(Psi_RtkFK)));
        FCC=[FCC1 FCC2]; % FCC Result 
        
%% [3-5] Computing Gamma_k(s(t)) analytically     
        vvC=[]; for i=1:S
                    InvXiC=Xi{termK,i}(1:n,1:n)\C{i,1};
                    vvC=[vvC;InvXiC(:)]; 
                end
        vvGamma=(eye(S*n*m)-Psi_RtkFK)\vvC;

        vGamma=reshape(vvGamma,n*m,S);
        for i=1:S, GammaK{i,1}=reshape(vGamma(:,i),n,m); end
    % ** If R_Psi_RtkFK <1, then GammaK is the forward solution.
    % ** If R_Psi_RtkFK >=1, then GammaK is NOT the forward solution. 
      
%% [4] Additional Output
    varargout(1)={FCC};
    varargout(2)={OmegaK};
    varargout(3)={GammaK};
    varargout(4)={FK};
        OtherOutput.BarPsi_OKkOK=BarPsi_OKkOK;
        OtherOutput.Psi_FKkFK=Psi_FKkFK;
        OtherOutput.Psi_RtkFK=Psi_RtkFK;
        OtherOutput.maxK=maxK;
%        OtherOutput.H=H; SL
    varargout(5)={OtherOutput}; 

%% [5] Impulse-Responses Function under Regime-Switching
if nargout>5,    
    D12=diag(vecsig); 
        %FT{j,t) and GT{j,t} are the coefficients of x(t) and z(t) 
        %in E_0[(x(t)].
        % Construct FT{j,1) and GT{j,1)
                for i=1:S,
                    FT{i,1}=zeros(n,n); GT{i,1}=zeros(n,m); 
                    for j=1:S,                         
                        FT{i,1}=FT{i,1}+P(i,j)*OmegaK{j};
                        GT{i,1}=GT{i,1}+P(i,j)*GammaK{j}*RR{j}; 
                 end, end
      
            for t=2:T-1,
                for i=1:S,
                    FT{i,t}=zeros(n,n); GT{i,t}=zeros(n,m);
                    for j=1:S,                      
                        FT{i,t}=FT{i,t}+P(i,j)*FT{j,t-1}*OmegaK{j};
                        GT{i,t}=GT{i,t}+P(i,j)*((GT{j,t-1}+FT{j,t-1}...
                                *GammaK{j})*RR{j}); 
                end, end
            end
        
        for i=1:S,
                if vs==1, 
                    tmp=GammaK{i}*D12; IRs=tmp(:)';
                        for t=1:T-1, temp=(FT{i,t}*GammaK{i}+GT{i,t})*D12; 
                            IRs=[IRs;temp(:)']; 
                        end
                    IRS{i,1}=IRs;
                else % Default
                    tmp=(GammaK{i}*D12)'; IRs=tmp(:)';
                        for t=1:T-1, 
                            temp=((FT{i,t}*GammaK{i}+GT{i,t})*D12)'; 
                            IRs=[IRs;temp(:)']; 
                        end
                    IRS{i,1}=IRs;    
                end
        end               
    varargout(6)={IRS};
end