function [y, lik_mat, pr_st0, pr_stl0] = lik_ms_sbvar(xdd, A, F, E, Q, nstates, nlags)
% Compute the sample log-likelihood function
% Markov-switching structural-form VAR form 
%
% Stephane Lhuissier 2015

t    = size(xdd,1);
nvar = size(xdd,2);
nexo = 1;

%[xtx,xty,yty,fss,xx,yy,ncoef,xr,Bh,e] = fn_dataxy(nvar,nlags,xdd,0,0,1);

nSample = size(xdd,1); % the sample size (including lags, of course)
sb = nlags+1;   % original beginning without dummies
ncoef = nvar*nlags+nexo;  % number of coefficients in *each* equation, RHS coefficients only.

   fss = nSample-nlags;
   %
   % ** construct X for Y = X*B + U where phi = X: (T-lags)*k, Y: (T-lags)*nvar
   % **    columns: k = # of [nvar for 1st lag, ..., nvar for last lag, exo var, const]
   %
   phi = zeros(fss,ncoef);
   %* constant term
   const = ones(fss,1);
   phi(:,ncoef) = const;      % the first nvar periods: no or zero constant!
   %* other exogenous (than) constant term
   phi(:,ncoef-nexo+1:ncoef-1) = xdd(nlags+1:end,nvar+1:nvar+nexo-1);
            % this = [] when nexo=1 (no other exogenous than constant)

   xdgel = xdd(:,1:nvar);  % endogenous variable matrix
   %* True data
   for k=1:nlags
      phi(:,nvar*(k-1)+1:nvar*k) = xdgel(sb-k:nSample-k,:);
      % row: T-lags; column: [nvar for 1st lag, ..., nvar for last lag, exo var, const]
      %                     Thus, # of columns is nvar*lags+nexo = ncoef.
   end
   %
   yy = xdgel(sb:nSample,:);
   xx = phi;


% Initializing Markov-switching framework
%--------------------------------------------------------------
%Steady-state probabilities PR[S_t=0] and PR[S_t=1]
Prob_ss = 1/nstates*ones(nstates,1);
Prob    = vecr(Prob_ss(:,ones(1,nstates))); % A 4x1 matrix

pr_st0  = zeros(t-nlags,nstates); % save all Pr[S_t=0|Y_{t}]
pr_stl0 = zeros(t-nlags,nstates); % save all Pr[S_t=0|Y_{t-1}]

% The sample log-likelihood function
%--------------------------------------------------------------
lik_mat = zeros(t-nlags,1);
lcond   = zeros(nstates,1);
detA    = zeros(nstates,1);
fss     = -0.5*nvar*log(2*pi);

for i=1:nstates
	[A0l,A0u] = lu(A(:,:,i));
    detA(i,1) = sum(log(abs(diag(A0u))));
end

for k=1:t-nlags

    Prob_j = Q(:).*Prob; % A 4x1 vector  
  
	for j=1:nstates
	block_cond = 0;

		for i=1:nvar
			block_cond = block_cond + log(abs(E(i,j))) - E(i,j)^2*0.5*(yy(k,:)*A(:,i,j) - xx(k,:)*F(:,i,j))^2;
		end
	
	    lcond(j,:) = detA(j,1) + block_cond;
	end

	vec_lcond = lcond(:,ones(1,nstates));

	%conditional densities, weight being Pr[S_{t}=j,S_{t-1}=i|Y_{t-1}]
	densj = fss + vec_lcond(:) + log(Prob_j);


	%log of Weighted average of conditional densities Pr[Y_t|Y_{t-1}] --
	maxdensj = max(densj);
	dens = log(sum(exp(densj - maxdensj))) + maxdensj;

	%log likelihood for each date
	lik_mat(k,1) = dens;

	%Update the probability terms Pr[S_t,S_{t-1}|Y_t] -----------
	JointProb = exp(densj - dens);

	%Update the probability terms Pr[S_t|Y_t] -------------------
	Pr = sum(reshape(JointProb,nstates,nstates),2);

	%build the probability terms Prob for the next period
	Prob = vecr(Pr(:,ones(1,nstates)));

	%get the probability terms Pr[S_t=0|Y_{t-1}] for Kim filter
	tmp = sum(reshape(Prob_j,nstates,nstates),2);

	pr_st0(k,:)  = Pr';  % Pr[S_t=0|Y_t]
	pr_stl0(k,:) = tmp'; % Pr[S_t=0|Y_{t-1}]

end

y = sum(lik_mat(:,1));
