% Main file for estimating and simulating MS-DSGE model
% using a MS-SVAR-implied impulse responses matching approach
%
% "Regime-dependent Effects of Uncertainty Shocks: A Structural
% Interpretation", forthcoming at Quantitative Economics
%
% by Stephane LHUISSIER and Fabien TRIPIER

clear all; clc; close all;

addpath(genpath('../'))
addpath(genpath('../../library/'))

% =========================================================================
% STEP 0. LOAD DATA AND PARAMETERS 
% =========================================================================

% name of file
name_file                = 'baseline';

% some flags
flag_fig                 = 0; % Plot figure during estimation/simulation
flag_mode                = 1; % Find Posterior mode
flag_hess                = 1; % Compute Hessian at the mode
flag_rwmh                = 1; % Run MCMC

% recover output from Dynare 
load('cmr_results','M_','oo_','options_');

% IRFs from estimated MS-VAR model
get_MS_VAR_resp;

% Parameters (initial values, prior, bounds, ...)
eval(strcat('prio_',name_file));
options_.ms.pshape       = prior(:,1);
options_.ms.sp           = prior(:,2);
options_.ms.pmean        = prior(:,3);
options_.ms.pstdd        = prior(:,4);
options_.ms.mask         = prior(:,5);
options_.ms.maskinv      = 1-options_.ms.mask;
options_.ms.fix          = prior(:,6);
options_.ms.trspec       = prior(:,7:10);
options_.ms.bounds       = options_.ms.trspec(:,2:3);
options_.ms.lb           = options_.ms.trspec(:,2);
options_.ms.ub           = options_.ms.trspec(:,3);
options_.ms.poscal       = find(options_.ms.mask); 
options_.ms.posest       = (1:length(options_.ms.sp)); 
options_.ms.posest       = find(ismember(options_.ms.posest(:),options_.ms.poscal(:))==0);

% options for Chris Sims's Csminwel
H0                  = 1e-4*eye(length(options_.ms.posest));
crit                = 1.0e-6;
nit                 = 10000;
gradient_method     = 2;
gradient_epsilon    = 1e-6;

%initialize parameters at fixed value or 0   
para                = options_.ms.sp.*(1-options_.ms.mask)+options_.ms.fix.*options_.ms.mask;

% evaluation of the log-posterior at the initial values
lnlikelihood        = compute_likelihood(para, M_, oo_, options_, flag_fig)
lnprior             = priodens(options_.ms.sp(options_.ms.posest), options_.ms.pmean(options_.ms.posest), options_.ms.pstdd(options_.ms.posest), options_.ms.pshape(options_.ms.posest))

% create results folder
if ~exist(['results/',name_file], 'dir')
    mkdir(['results/',name_file])
end
    
% =========================================================================
% STEP 1. OPTIMIZATION ROUTINE
% =========================================================================

if (flag_mode)
    tic 
    [fh,xh,gh,H,itct,fcount,retcodeh] = csminwel1('objfun',invtrans(options_.ms.sp(options_.ms.posest), ...
                         options_.ms.trspec(options_.ms.posest,:)),H0,[],crit,nit,  ...
                         gradient_method, gradient_epsilon, M_, oo_, options_,      ...
                         1, flag_fig)

    % display computing time	
    fprintf(1,'\nTotal CSMINWEL computing time: %s\n',secs2hms(toc));

    xh    = trans(xh,options_.ms.trspec(options_.ms.posest,:));

    disp(' ')
    disp('----------------------------------------------------------------')
    disp('RESULTS [Posterior mode]')
    disp('----------------------------------------------------------------')
    cnames={'mode','grad','mean (prior)','std (prior)','type (prior)'};
    pnames=para_names(options_.ms.posest)';
    r=printresults3([xh,abs(gh),options_.ms.pmean(options_.ms.posest),options_.ms.pstdd(options_.ms.posest),options_.ms.pshape(options_.ms.posest)],cnames,pnames); disp(r)

    % posterior mode 
    Postmode                        = para;
    Postmode(options_.ms.posest)    = xh;

    % evaluation of the log-likelihood at the initial values
    lnlikelihood        = compute_likelihood(Postmode, M_, oo_, options_, 1);
    lnprior             = priodens(Postmode(options_.ms.posest), options_.ms.pmean(options_.ms.posest), options_.ms.pstdd(options_.ms.posest), options_.ms.pshape(options_.ms.posest));
    lnposterior         = real(lnlikelihood+lnprior);

    % save main results
    save(['results/',name_file,'/posterior_mode.mat'],'Postmode','lnlikelihood','lnprior','lnposterior','xh','pnames');
end

% =========================================================================
% STEP 2. COMPUTE HESSIAN AT POSTERIOR MODE
% =========================================================================
if (flag_hess)
    % load posterior mode
    if ~flag_mode
       load(['results/',name_file,'/posterior_mode.mat'],'Postmode'); 
    end    
    
    % Compute Hessian at mode
    hessian = hessizero('compute_lnposterior',[Postmode(options_.ms.posest), zeros(length(options_.ms.posest),1)], 1, M_, oo_, options_, 0);

    % Create Inverse by Singular Value Decomposition
    [u,s,v] = svd(hessian);
    md = min(size(s)); 
    bigev = find(diag(s(1:md,1:md))>1e-12);
    sigpropdim = length(bigev);

    sigproplndet = 0;
    for i = 1:length(options_.ms.posest)

      if i > sigpropdim
        s(i,i) = 0;
      else

        s(i,i)    = 1/s(i,i);
        sigproplndet = sigproplndet+log(s(i,i));  

      end
    end
    invhhm  = u*s*u';
    sigscale = u*sqrt(s);
    sigpropinv  =  hessian; 
    clear hessian*;

    save(['results/',name_file,'/hessian.mat'],'sigscale');
end

% =========================================================================
% STEP 3. MCMC
% =========================================================================
if (flag_rwmh)
    % load hessian
    if ~flag_hess
       load(['results/',name_file,'/hessian.mat'],'sigscale'); 
    end
    
    % load posterior mode
    if ~flag_mode
       load(['results/',name_file,'/posterior_mode.mat'],'xh'); 
    end    
    
    % options
    ndraws       = 50000;
    scaling      = 0.40;%22;%28; %0.32;
    seednumber   = [122; 756; 345; 582];

    randn('state',seednumber(1));
    rand('state',seednumber(1));
    
    options_.steadystate.nocheck = 1;

    tic
    paraMCMC = fn_rwmh(name_file,xh,scaling,sigscale,1,ndraws,M_,oo_,options_,flag_fig);

    % display computing time	
    fprintf(1,'Total MCMC computing time: %s\n\n',secs2hms(toc));

    % compute posterior moments (median, mean, and 90% prob. intervals)
    Postdraw_med = median(paraMCMC,1);
    Postdraw_mea = mean(paraMCMC,1);
    Postdraw_lim = hpdint(paraMCMC,0.90,0);

    disp(' ')
    disp('----------------------------------------------------------------')
    disp('RESULTS [Posterior distribution]')
    disp('----------------------------------------------------------------')
    cnames={'Mode','Mean','Median','Lower', 'Upper'};
    out=[xh,Postdraw_mea',Postdraw_med',Postdraw_lim'];
    r=printresults3(out,cnames,para_names(options_.ms.posest)'); disp(r)
end

